#-*- perl -*-

package Unicode::LineBreak;

=encoding utf8

This file is automatically generated.  DON'T EDIT THIS FILE MANUALLY.

=cut

our $VERSION = '1.003_05';
use constant { UNICODE_VERSIONS => [qw{5.1.0 5.2.0beta}] };
use constant { DEFAULT_UNICODE_VERSION => '5.1.0' };

1;
